//  Shapes.cpp
////////////////////////////////////////
// 
#include "stdafx.h"
#include "shapes.h"



CBasePoint::CBasePoint(int x, int y)
{
	m_dScale=1.0;
	m_x=x;
	m_y=y;
};

CBasePoint::CBasePoint()
{
	m_dScale=1.0;
	m_x=100;
	m_y=100;
};

void CBasePoint::Show(CDC* pDC) 
{
	int r=1;
	CBrush *pOldBrush=pDC->SelectObject(&CBrush(RGB(100,100,100)));
	pDC->Ellipse(m_x-r, m_y-r, m_x+r, m_y+r);
	pDC->SelectObject(pOldBrush);
} 

void CBasePoint::GetRegion(CRgn &Rgn)
{
	int r=1;
	Rgn.CreateEllipticRgn(m_x-r, m_y-r, m_x+r, m_y+r);
}

IMPLEMENT_SERIAL(CBasePoint, CObject , 1)
void CBasePoint::Serialize(CArchive &ar)
{
	if(ar.IsStoring())
	{
		ar<<m_x;
		ar<<m_y;
		ar<<m_dScale;
	}
	else
	{
		ar>>m_x;
		ar>>m_y;
		ar>>m_dScale;
	}
};


CCircle::CCircle(int x, int y, WORD r): CBasePoint(x, y)
{
	m_wRadius=r;
} 

CCircle::CCircle(): CBasePoint()
{
	m_wRadius=25;
} 

void CCircle::Show(CDC* pDC) 
{
	int r=m_dScale*m_wRadius;
	CBrush *pOldBrush=pDC->SelectObject(&CBrush(HS_CROSS, RGB(200,0,0)));
	pDC->Ellipse(m_x-r, m_y-r, m_x+r, m_y+r);
	pDC->SelectObject(pOldBrush);
} 

void CCircle::GetRegion(CRgn &Rgn)
{
	int r=m_dScale*m_wRadius;
	Rgn.CreateEllipticRgn(m_x-r, m_y-r, m_x+r, m_y+r);
}

IMPLEMENT_SERIAL(CCircle, CBasePoint, 1)
void CCircle::Serialize(CArchive &ar)
{
	if(ar.IsStoring())
	{
		ar<<m_wRadius;		
	}
	else
	{
		ar>>m_wRadius;
	}
	CBasePoint::Serialize(ar);
};


CSquare::CSquare(int x, int y, WORD s): CBasePoint(x, y)
{
	m_wSide=s;
} 

CSquare::CSquare(): CBasePoint()
{
	m_wSide=40;
} 

void CSquare::Show(CDC* pDC) 
{
	int s=m_dScale*m_wSide/2;
	CBrush *pOldBrush=pDC->SelectObject(&CBrush(HS_DIAGCROSS, RGB(0,200,0)));
	pDC->Rectangle(m_x-s, m_y-s, m_x+s, m_y+s);
	pDC->SelectObject(pOldBrush);
} 

void CSquare::GetRegion(CRgn &Rgn)
{
	int s=m_dScale*m_wSide/2;
	Rgn.CreateRectRgn(m_x-s, m_y-s, m_x+s, m_y+s);
}

IMPLEMENT_SERIAL(CSquare, CBasePoint, 1)
void CSquare::Serialize(CArchive &ar)
{
	if(ar.IsStoring())
	{
		ar<<m_wSide;		
	}
	else
	{
		ar>>m_wSide;
	}
	CBasePoint::Serialize(ar);
};
