//  Shapes.cpp
////////////////////////////////////////
// 
#include "stdafx.h"
#include "shapes.h"
#include "math.h"


CBasePoint::CBasePoint(int x, int y)
{
	m_dScale=1.0;
	m_x=x;
	m_y=y;
};

CBasePoint::CBasePoint()
{
	m_dScale=1.0;
	m_x=100;
	m_y=100;
};

void CBasePoint::Show(CDC* pDC) 
{
	int r=1;
	CBrush *pOldBrush=pDC->SelectObject(&CBrush(RGB(100,100,100)));
	pDC->Ellipse(m_x-r, m_y-r, m_x+r, m_y+r);
	pDC->SelectObject(pOldBrush);
} 

void CBasePoint::GetRegion(CRgn &Rgn)
{
	int r=1;
	Rgn.CreateEllipticRgn(m_x-r, m_y-r, m_x+r, m_y+r);
}

CPoint CBasePoint::Transform(	const CPoint &point,
				const CPoint &point0, double ang, int a, int b)
{
	CPoint res;
	res.x=point0.x+(point.x-point0.x)*cos(ang)-(point.y-point0.y)*sin(ang)+a;
	res.y=point0.y+(point.x-point0.x)*sin(ang)+(point.y-point0.y)*cos(ang)+b;
	return res;
};

IMPLEMENT_SERIAL(CBasePoint, CObject , 1)
void CBasePoint::Serialize(CArchive &ar)
{
	if(ar.IsStoring())
	{
		ar<<m_x;
		ar<<m_y;
		ar<<m_dScale;
	}
	else
	{
		ar>>m_x;
		ar>>m_y;
		ar>>m_dScale;
	}
};


CCircle::CCircle(int x, int y, WORD r): CBasePoint(x, y)
{
	m_wRadius=r;
} 

CCircle::CCircle(): CBasePoint()
{
	m_wRadius=25;
} 

void CCircle::Show(CDC* pDC) 
{
	int r=m_dScale*m_wRadius;
	CBrush *pOldBrush=pDC->SelectObject(&CBrush(HS_CROSS, RGB(200,0,0)));
	pDC->Ellipse(m_x-r, m_y-r, m_x+r, m_y+r);
	pDC->SelectObject(pOldBrush);
} 

void CCircle::GetRegion(CRgn &Rgn)
{
	int r=m_dScale*m_wRadius;
	Rgn.CreateEllipticRgn(m_x-r, m_y-r, m_x+r, m_y+r);
}

IMPLEMENT_SERIAL(CCircle, CBasePoint, 1)
void CCircle::Serialize(CArchive &ar)
{
	if(ar.IsStoring())
	{
		ar<<m_wRadius;		
	}
	else
	{
		ar>>m_wRadius;
	}
	CBasePoint::Serialize(ar);
};


CSquare::CSquare(int x, int y, WORD s): CBasePoint(x, y)
{
	m_wSide=s;
} 

CSquare::CSquare(): CBasePoint()
{
	m_wSide=40;
} 

void CSquare::Show(CDC* pDC) 
{
	int s=m_dScale*m_wSide/2;
	CBrush *pOldBrush=pDC->SelectObject(&CBrush(HS_DIAGCROSS, RGB(0,200,0)));
	pDC->Rectangle(m_x-s, m_y-s, m_x+s, m_y+s);
	pDC->SelectObject(pOldBrush);
} 

void CSquare::GetRegion(CRgn &Rgn)
{
	int s=m_dScale*m_wSide/2;
	Rgn.CreateRectRgn(m_x-s, m_y-s, m_x+s, m_y+s);
}

IMPLEMENT_SERIAL(CSquare, CBasePoint, 1)
void CSquare::Serialize(CArchive &ar)
{
	if(ar.IsStoring())
	{
		ar<<m_wSide;		
	}
	else
	{
		ar>>m_wSide;
	}
	CBasePoint::Serialize(ar);
};

////////////////////////////////////////////////////
// 
CPoligon::CPoligon(): CBasePoint()
{
	m_nIndex=0;
};

CPoligon::~CPoligon()
{

};
void CPoligon::Show(CDC* pDC) 
{
	
	CBrush *pOldBrush=pDC->SelectObject(&CBrush(HS_DIAGCROSS, RGB(0,0,200)));
	
	pDC->Polygon( m_Points, m_nIndex);
		
	pDC->SelectObject(pOldBrush);
} 

void CPoligon::GetRegion(CRgn &Rgn)
{
	Rgn.CreatePolygonRgn( m_Points, m_nIndex, WINDING);
}
BOOL CPoligon::AddPoint(CPoint point)
{
	if(m_nIndex<MAXPOLIGONLENGTH)
	{
		m_Points[m_nIndex++]=point;
	}
	else return FALSE;
	return TRUE;
};

IMPLEMENT_SERIAL(CPoligon, CBasePoint, 1)
void CPoligon::Serialize(CArchive &ar)
{
	if(ar.IsStoring())
	{
		ar << m_nIndex;
		for(int i=0; i<m_nIndex; i++) ar << m_Points[i];
	}
	else
	{
		ar >> m_nIndex;
		for(int i=0; i<m_nIndex; i++) ar >> m_Points[i];
	}
	
	CBasePoint::Serialize(ar);
};


