// Paintvw.cpp : implementation of the CPainterView class
//

#include "stdafx.h"
#include "Painter.h"

#include "Paintdoc.h"
#include "Paintvw.h"
#include "Shapes.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPainterView

IMPLEMENT_DYNCREATE(CPainterView, CView)

BEGIN_MESSAGE_MAP(CPainterView, CView)
	//{{AFX_MSG_MAP(CPainterView)
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_COMMAND(ID_SHAPES_SQUARE, OnShapesSquare)
	ON_UPDATE_COMMAND_UI(ID_SHAPES_SQUARE, OnUpdateShapesSquare)
	ON_COMMAND(ID_SHAPES_CIRCLE, OnShapesCircle)
	ON_UPDATE_COMMAND_UI(ID_SHAPES_CIRCLE, OnUpdateShapesCircle)
	ON_COMMAND(ID_SHAPES_POLIGON, OnShapesPoligon)
	ON_UPDATE_COMMAND_UI(ID_SHAPES_POLIGON, OnUpdateShapesPoligon)
	ON_WM_LBUTTONDBLCLK()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_COMMAND(ID_SHAPES_CUBE, OnShapesCube)
	ON_UPDATE_COMMAND_UI(ID_SHAPES_CUBE, OnUpdateShapesCube)
	ON_COMMAND(ID_SHAPES_SELECT, OnShapesSelect)
	ON_UPDATE_COMMAND_UI(ID_SHAPES_SELECT, OnUpdateShapesSelect)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPainterView construction/destruction

CPainterView::CPainterView()
{
	// TODO: add construction code here
	m_CurOper=m_LastOper=OP_NOOPER;
}

CPainterView::~CPainterView()
{
}

/////////////////////////////////////////////////////////////////////////////
// CPainterView drawing

void CPainterView::OnDraw(CDC* pDC)
{
	CPainterDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
	if(pDoc->m_nIndex>0) pDC->MoveTo(pDoc->m_Points[0]);
	for(int i=1;i<pDoc->m_nIndex; i++)
		pDC->LineTo(pDoc->m_Points[i]);

	POSITION pos=NULL;
	CBasePoint* pShape=NULL;
	if(pDoc->m_ShapesList.GetCount()>0) pos=pDoc->m_ShapesList.GetHeadPosition();
	while(pos!=NULL)
	{
		pShape=pDoc->m_ShapesList.GetNext(pos);
		if(pShape!=NULL) pShape->Show(pDC);
	}
	// 
	if(m_CurOper==OP_DRAWPOLIGON && m_pCurShape!=NULL)
	m_pCurShape->Show(pDC);

}

/////////////////////////////////////////////////////////////////////////////
// CPainterView printing

BOOL CPainterView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CPainterView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CPainterView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CPainterView diagnostics

#ifdef _DEBUG
void CPainterView::AssertValid() const
{
	CView::AssertValid();
}

void CPainterView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CPainterDoc* CPainterView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CPainterDoc)));
	return (CPainterDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CPainterView message handlers

void CPainterView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CPainterDoc *pDoc=GetDocument();
	switch(m_CurOper)
	{
	case OP_NOOPER:
		if(pDoc->m_nIndex==MAXPOINTS)	
			AfxMessageBox("  ");
		else
		{
			pDoc->m_Points[pDoc->m_nIndex++]=point;
			Invalidate();
			pDoc->SetModifiedFlag();
		}
		m_LastOper=m_CurOper;
		m_CurOper=OP_NOOPER;
	break;

	case OP_SELECT:
		pDoc->SelectShape(point);
	break;

	case OP_DRAWCIRCLE:
		pDoc->AddShape(new CCircle(point, 20));
		Invalidate();
		pDoc->SetModifiedFlag();
		m_LastOper=m_CurOper;
		m_CurOper=OP_NOOPER;
    break;
	case OP_DRAWSQUARE:
		pDoc->AddShape(new CSquare(point, 40));
		Invalidate();
		pDoc->SetModifiedFlag();
		m_LastOper=m_CurOper;
		m_CurOper=OP_NOOPER;
	break;
	case OP_DRAWPOLIGON:
		if(m_pCurShape!=NULL) ((CPoligon*)m_pCurShape)->AddPoint(point);
		Invalidate();
	break;	
	case OP_DRAWCUBE:
		pDoc->AddShape(new CCube(point, 100));	
		Invalidate();	pDoc->SetModifiedFlag();
		m_LastOper=m_CurOper; m_CurOper=OP_NOOPER;
	break;

	}
	CView::OnLButtonDown(nFlags, point);
}

void CPainterView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CPainterDoc *pDoc=GetDocument();
	switch(m_LastOper)
	{
	case OP_NOOPER:
		if(pDoc->m_nIndex>0) pDoc->m_nIndex--;
		Invalidate();
		pDoc->SetModifiedFlag();
	break;
	case OP_DRAWCIRCLE:
	case OP_DRAWSQUARE:
	case OP_DRAWPOLIGON:
		if(pDoc->DeleteLastShape())
		{
			Invalidate();
			pDoc->SetModifiedFlag();
		}
	break;
	}

	Invalidate();
	pDoc->SetModifiedFlag();
	CView::OnRButtonDown(nFlags, point);
}

void CPainterView::OnShapesSquare() 
{
	m_CurOper=OP_DRAWSQUARE;
}

void CPainterView::OnUpdateShapesSquare(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_CurOper==OP_DRAWSQUARE);
}

void CPainterView::OnShapesCircle() 
{
	m_CurOper=OP_DRAWCIRCLE;
}

void CPainterView::OnUpdateShapesCircle(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_CurOper==OP_DRAWCIRCLE);
}

void CPainterView::OnShapesPoligon() 
{
	// TODO: Add your command handler code here
	m_pCurShape=new CPoligon();
	m_CurOper=OP_DRAWPOLIGON;
}

void CPainterView::OnUpdateShapesPoligon(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_CurOper==OP_DRAWPOLIGON);
}

void CPainterView::OnShapesCube() 
{
	// TODO: Add your command handler code here
	m_CurOper=OP_DRAWCUBE;	
}

void CPainterView::OnUpdateShapesCube(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_CurOper==OP_DRAWCUBE);	
}

void CPainterView::OnShapesSelect() 
{
	if(m_CurOper==OP_SELECT) m_CurOper=OP_NOOPER;
	else m_CurOper=OP_SELECT;
}

void CPainterView::OnUpdateShapesSelect(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_CurOper==OP_SELECT);	
}

void CPainterView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	if(m_CurOper==OP_DRAWPOLIGON)
	{
		GetDocument()->AddShape(m_pCurShape);	
		GetDocument()->SetModifiedFlag();
		m_LastOper=m_CurOper;
		m_CurOper=OP_NOOPER;
	}
	CView::OnLButtonDblClk(nFlags, point);
}

void CPainterView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	GetDocument()->OnKeyDown(nChar, nRepCnt, nFlags);
	Invalidate();
	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}


void CPainterView::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	GetDocument()->OnKeyUp(nChar, nRepCnt, nFlags);
	CView::OnKeyUp(nChar, nRepCnt, nFlags);
}


