//Shapes.cpp
////////////////////////////////////////
// 
#include "stdafx.h"
#include "shapes.h"
#include "math.h"



CBasePoint::CBasePoint(CPoint point)
{
	m_dScale=1.0;
	m_x=point.x;
	m_y=point.y;
	SetColor(RGB(100,100,100));
	SetStep(5);
	m_FlagShift=FALSE;
	m_FlagCtrl=FALSE;
};

CBasePoint::CBasePoint()
{
	m_dScale=1.0;
	m_x=100;
	m_y=100;
	SetColor(RGB(100,100,100));
	SetStep(5);
	m_FlagShift=FALSE;
	m_FlagCtrl=FALSE;
};

CPoint CBasePoint::Transform(const CPoint &point,
	       const CPoint &point0, double ang, int a, int b)
{
   CPoint res;
   	//  
	ang=ang*atan(1.0)/45.0;
   res.x=point0.x+(point.x-point0.x)*cos(ang)-(point.y-point0.y)*sin(ang)+a;
   res.y=point0.y+(point.x-point0.x)*sin(ang)+(point.y-point0.y)*cos(ang)+b;
   return res;
};


void CBasePoint::Show(CDC* pDC) 
{
	int r=1;
	CBrush	Brush(m_Color);
	CBrush	*pOldBrush=pDC->SelectObject(&Brush);
	
	pDC->Ellipse(m_x-r, m_y-r, m_x+r, m_y+r);

	pDC->SelectObject(pOldBrush);
} 

void CBasePoint::GetRegion(CRgn &Rgn)
{
	int r=1;
	Rgn.CreateEllipticRgn(m_x-r, m_y-r, m_x+r, m_y+r);
}

BOOL CBasePoint::PointInShape(CPoint point)
{
	CRgn Rgn;
	GetRegion(Rgn);
	return Rgn.PtInRegion(point);
};

void CBasePoint::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	switch(nChar)
	{
		case 16: m_FlagShift=TRUE; break; //Shift
		case 17: m_FlagCtrl=TRUE; break; //Ctrl
		case 38: m_y-=m_Step; break;	  //Up
		case 40: m_y+=m_Step; break;	  //Down
		case 37: m_x-=m_Step; break;	  //Left
		case 39: m_x+=m_Step; break;	  //Right
	}
};
void CBasePoint::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	switch(nChar)
	{
		case 16: m_FlagShift=FALSE; break; //Shift
		case 17: m_FlagCtrl=FALSE; break; //Ctrl
	}

}

IMPLEMENT_SERIAL(CBasePoint, CObject , 1)
void CBasePoint::Serialize(CArchive &ar)
{
	DWORD temp=0;m_Color;
	if(ar.IsStoring())
	{
		ar<<m_x;
		ar<<m_y;
		ar<<m_dScale;
		temp=m_Color;
		ar<<temp;
	}
	else
	{
		ar>>m_x;
		ar>>m_y;
		ar>>m_dScale;
		ar>>temp;
		m_Color=temp;
	}
};


CCircle::CCircle(CPoint point, WORD r): CBasePoint(point)
{
	m_wRadius=r;
	SetColor(RGB(200,0,0));
} 

CCircle::CCircle(): CBasePoint()
{
	m_wRadius=25;
	SetColor(RGB(200,0,0));
} 

void CCircle::Show(CDC* pDC) 
{
	int r=m_dScale*m_wRadius;
	CBrush	Brush(HS_CROSS, m_Color);
	CBrush	*pOldBrush=pDC->SelectObject(&Brush);
	
	pDC->Ellipse(m_x-r, m_y-r, m_x+r, m_y+r);

	pDC->SelectObject(pOldBrush);
} 

void CCircle::GetRegion(CRgn &Rgn)
{
	int r=m_dScale*m_wRadius;
	Rgn.CreateEllipticRgn(m_x-r, m_y-r, m_x+r, m_y+r);
}

IMPLEMENT_SERIAL(CCircle, CBasePoint, 1)
void CCircle::Serialize(CArchive &ar)
{
	if(ar.IsStoring())
	{
		ar<<m_wRadius;		
	}
	else
	{
		ar>>m_wRadius;
	}
	CBasePoint::Serialize(ar);
};


CSquare::CSquare(CPoint point, WORD s): CBasePoint(point)
{
	m_wSide=s;
	SetColor(RGB(0,200,0));
} 

CSquare::CSquare(): CBasePoint()
{
	m_wSide=40;
	SetColor(RGB(0,200,0));
} 

void CSquare::Show(CDC* pDC) 
{
	int s=m_dScale*m_wSide/2;
	CBrush Brush(HS_CROSS, m_Color);
	CBrush	*pOldBrush=pDC->SelectObject(&Brush);

	pDC->Rectangle(m_x-s, m_y-s, m_x+s, m_y+s);

	pDC->SelectObject(pOldBrush);
} 

void CSquare::GetRegion(CRgn &Rgn)
{
	int s=m_dScale*m_wSide/2;
	Rgn.CreateRectRgn(m_x-s, m_y-s, m_x+s, m_y+s);
}

IMPLEMENT_SERIAL(CSquare, CBasePoint, 1)
void CSquare::Serialize(CArchive &ar)
{
	if(ar.IsStoring())
	{
		ar<<m_wSide;		
	}
	else
	{
		ar>>m_wSide;
	}
	CBasePoint::Serialize(ar);
};
///////////////////////////////////
CPoligon::CPoligon(): CBasePoint()
{
	m_nIndex=0;
	SetColor(RGB(0,0,200));
};

CPoligon::CPoligon(CPoint point): CBasePoint(point)
{
	m_nIndex=0;
	SetColor(RGB(0,0,200));
}

CPoligon::~CPoligon()
{

};

void CPoligon::Show(CDC* pDC) 
{
	CBrush Brush(HS_DIAGCROSS, m_Color);
	CBrush	*pOldBrush=pDC->SelectObject(&Brush);

	pDC->Polygon( m_Points, m_nIndex);
		
	pDC->SelectObject(pOldBrush);
} 

void CPoligon::GetRegion(CRgn &Rgn)
{
	Rgn.CreatePolygonRgn( m_Points, m_nIndex, WINDING);
}

BOOL CPoligon::AddPoint(CPoint point)
{
	if(m_nIndex<MAXPOLIGONLENGTH)
	{
		m_Points[m_nIndex++]=point;
	}
	else return FALSE;
	for(int i=0; i<m_nIndex; i++)
	{
		m_x+=m_Points[i].x;
		m_y+=m_Points[i].y;
	}
	m_x/=m_nIndex;	m_y/=m_nIndex;
	return TRUE;
};

void CPoligon::Transform(double ang, int a, int b)
{
	for(int i=0; i<m_nIndex; i++)
	{
		m_Points[i]=CBasePoint::Transform(m_Points[i], m_Points[0], ang, a, b);
	}
};
void CPoligon::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	
	if(m_FlagShift) //
	switch(nChar)
	{
		case 37: Transform(-m_Step, 0, 0); break;
		case 39: Transform(m_Step, 0, 0); break;
	}
	else		   //
	switch(nChar)
	{
		case 38: Transform(0, 0, -m_Step); break; //Up
		case 40: Transform(0, 0, m_Step); break;//Down
		case 37: Transform(0, -m_Step, 0); break;//Left
		case 39: Transform(0, m_Step, 0); break;//Right
	}

	CBasePoint::OnKeyDown( nChar, nRepCnt, nFlags);
};

IMPLEMENT_SERIAL(CPoligon, CBasePoint, 1)
void CPoligon::Serialize(CArchive &ar)
{
	if(ar.IsStoring())
	{
		ar << m_nIndex;
		for(int i=0; i<m_nIndex; i++) ar << m_Points[i];
	}
	else
	{
		ar >> m_nIndex;
		for(int i=0; i<m_nIndex; i++) ar >> m_Points[i];
	}
	
	CBasePoint::Serialize(ar);
};

/////////////////////////////////////////////////
//3D Poligon

C3DPoligon::C3DPoligon():CPoligon()
{
};

C3DPoligon::C3DPoligon(CPoint point):CPoligon(point)
{
};


C3DPoligon::~C3DPoligon()
{

};


IMPLEMENT_SERIAL(C3DPoligon, CPoligon, -1)
void C3DPoligon::Serialize(CArchive &ar)
{
	CPoligon::Serialize(ar);
	
	if(ar.IsStoring())
	{
		for(int i=0; i<m_nIndex; i++)
		{ 	ar << m_3DPoints[i].x;
			ar << m_3DPoints[i].y;
			ar << m_3DPoints[i].z;
		}
	}
	else
	{
		for(int i=0; i<m_nIndex; i++)
		{ 	ar >> m_3DPoints[i].x;
			ar >> m_3DPoints[i].y;
			ar >> m_3DPoints[i].z;
		}
	}
	
};

void C3DPoligon::MakeProjection(Perspective P)
{
	//  
	P.theta=P.theta*atan(1.0)/45.0; P.phi=P.phi*atan(1.0)/45.0;
	//   
	//   MM_TEXT,    
	//   ,         
	//     (  Y)
	double  st=sin(P.theta), ct=cos(P.theta), sp=sin(P.phi), cp=cos(P.phi),
	v11=-st, v12=+cp*ct, v13=-sp*ct,
	v21=ct,  v22=+cp*st, v23=-sp*st,
			 v32=-sp,     v33=-cp,
									 v43=P.rho;
	double x, y, z;
	LONG TempZ=0;
	//   
	for(int i=0; i<m_nIndex; i++)
	{
		x=m_3DPoints[i].x-P.O.x;
		y=m_3DPoints[i].y-P.O.y;
		z=m_3DPoints[i].z-P.O.z;

		TempZ=v13*x+v23*y+v33*z+v43;
		m_Points[i].x=v11*x+v21*y;
		m_Points[i].y=v12*x+v22*y+v32*z;

		//  
		if(P.with_perspective)
		{
			m_Points[i].x=P.d*m_Points[i].x/TempZ;
			m_Points[i].y=P.d*m_Points[i].y/TempZ;
		}
		
		m_Points[i].x+=P.O.x;
		m_Points[i].y+=P.O.y;
		//     
		m_Points[i].x+=m_x;
		m_Points[i].y+=m_y;

	}
};

void C3DPoligon::Show(CDC *pDC)
{
	CPen Pen(PS_SOLID, 1, m_Color);
	CPen	*pOldPen=pDC->SelectObject(&Pen);

	if(m_nIndex>0) pDC->MoveTo(m_Points[0].x, m_Points[0].y);
	for(int i=1; i<m_nIndex; i++)
		pDC->LineTo(m_Points[i].x, m_Points[i].y);

	pDC->SelectObject(pOldPen);
};

BOOL C3DPoligon::AddPoint(POINT3D point)
{
	if(m_nIndex<MAXPOLIGONLENGTH)
	{
		m_3DPoints[m_nIndex++]=point;
	}
	else return FALSE;
	return TRUE;
};

//////////////////////////////////////////////////////
C3DShape::C3DShape(): CBasePoint()
{
	m_Percpective.O.x=0;
	m_Percpective.O.y=0;
	m_Percpective.O.z=0;
	m_Percpective.rho=100;
	m_Percpective.theta=30;
	m_Percpective.phi=30;
	m_Percpective.d=50;
	m_Percpective.with_perspective=TRUE;
}

C3DShape::C3DShape(CPoint point): CBasePoint(point)
{
	m_Percpective.O.x=0;
	m_Percpective.O.y=0;
	m_Percpective.O.z=0;
	m_Percpective.rho=500;
	m_Percpective.theta=30;
	m_Percpective.phi=30;
	m_Percpective.d=250;
	m_Percpective.with_perspective=TRUE;
}

C3DShape::~C3DShape()
{
	while(m_PtrPoligonList.GetCount()>0)
		delete m_PtrPoligonList.RemoveHead();
	
};

IMPLEMENT_SERIAL(C3DShape, CBasePoint, -1)
void C3DShape::Serialize(CArchive &ar)
{
	if(ar.IsStoring())
	{
		ar << m_Percpective.O.x;
		ar << m_Percpective.O.y;
		ar << m_Percpective.O.z;
		ar << m_Percpective.rho;
		ar << m_Percpective.theta;
		ar << m_Percpective.phi;
		ar << m_Percpective.d;
		ar << m_Percpective.with_perspective;
	}
	else
	{
		ar >> m_Percpective.O.x;
		ar >> m_Percpective.O.y;
		ar >> m_Percpective.O.z;
		ar >> m_Percpective.rho;
		ar >> m_Percpective.theta;
		ar >> m_Percpective.phi;
		ar >> m_Percpective.d;
		ar >> m_Percpective.with_perspective;
	}
	m_PtrPoligonList.Serialize(ar);
};

void C3DShape::Show(CDC *pDC)
{
	POSITION Pos=NULL;
	if(m_PtrPoligonList.GetCount()>0)
		Pos=m_PtrPoligonList.GetHeadPosition();
	
	while(Pos!=NULL)
		((C3DPoligon*)m_PtrPoligonList.GetNext(Pos))->Show(pDC);
};


BOOL C3DShape::PointInShape(CPoint point)
{
	POSITION Pos=NULL;
	if(m_PtrPoligonList.GetCount()>0)
		Pos=m_PtrPoligonList.GetHeadPosition();
	while(Pos!=NULL)
		if(((C3DPoligon*)m_PtrPoligonList.GetNext(Pos))->PointInShape(point))
			return TRUE; //     
	return FALSE;
};

	
void C3DShape::AddPoligon(C3DPoligon *pPoligon)
{
	m_PtrPoligonList.AddTail(pPoligon);	//  
	//  
	POSITION Pos=NULL;
	C3DPoligon* pCurPoligon=NULL;
	WORD Index=0, i=0;
	if(m_PtrPoligonList.GetCount()>0)
		Pos=m_PtrPoligonList.GetHeadPosition();
	while(Pos!=NULL)
	{
		pCurPoligon=(C3DPoligon*)m_PtrPoligonList.GetNext(Pos);
		for(i=0; i<pCurPoligon->m_nIndex; i++)
		{
			m_Percpective.O.x+=pCurPoligon->m_3DPoints[i].x;	
			m_Percpective.O.y+=pCurPoligon->m_3DPoints[i].y;	
			m_Percpective.O.z+=pCurPoligon->m_3DPoints[i].z;	
		}
		Index+=pCurPoligon->m_nIndex;
	}
	m_Percpective.O.x/=Index;
	m_Percpective.O.y/=Index;
	m_Percpective.O.z/=Index;
};


void C3DShape::MakeProjection()
{
	POSITION Pos=NULL;
	if(m_PtrPoligonList.GetCount()>0)
		Pos=m_PtrPoligonList.GetHeadPosition();
	while(Pos!=NULL)
		((C3DPoligon*)m_PtrPoligonList.GetNext(Pos))->MakeProjection(m_Percpective);
};

void C3DShape::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if(m_FlagShift)
	switch(nChar)
	{
		case 38: m_Percpective.phi-=m_Step; break;
		case 40: m_Percpective.phi+=m_Step; break;
		case 37: m_Percpective.theta-=m_Step; break;
		case 39: m_Percpective.theta+=m_Step; break;
	}
	else
	if(m_FlagCtrl)	 
	switch(nChar)
	{
		case 38: m_Percpective.d+=m_Step; break; //Up 	 
		case 40: m_Percpective.d-=m_Step; break; //Down  
	}
	else
	{
		CBasePoint::OnKeyDown( nChar, nRepCnt,  nFlags);
		POSITION Pos=NULL;
		if(m_PtrPoligonList.GetCount()>0)
			Pos=m_PtrPoligonList.GetHeadPosition();
		while(Pos!=NULL)
		((C3DPoligon*)m_PtrPoligonList.GetNext(Pos))->OnKeyDown( nChar, nRepCnt,  nFlags);
	}
	
	// 
	MakeProjection();

};

////////////////////////////////////////////////////
// CCube
IMPLEMENT_SERIAL(CCube, C3DShape, -1)
void CCube::Serialize(CArchive &ar)
{
	if(ar.IsStoring())
	{
		ar << m_wSide;
	}
	else
	{
		ar >> m_wSide;
	}
	C3DShape::Serialize(ar);
};

CCube::CCube():	C3DShape()
{
}

CCube::CCube(CPoint BasePoint, WORD side):	C3DShape(BasePoint)
{
	m_wSide=side;
	Create(BasePoint, m_wSide);
};

void CCube::Create(CPoint BasePoint, WORD side)
{
	m_x=BasePoint.x;
	m_y=BasePoint.y;
	POINT3D point;
	C3DPoligon* pPoligon=NULL;
	//  
	pPoligon=new C3DPoligon(BasePoint);
	if(!pPoligon) return;
	//1
	point.x=m_wSide/2; point.y=m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);
	//2
	point.x=m_wSide/2; point.y=m_wSide/2; point.z=m_wSide/2;
	pPoligon->AddPoint(point);
	//3
	point.x=m_wSide/2; point.y=-m_wSide/2; point.z=m_wSide/2;
	pPoligon->AddPoint(point);
	//4
	point.x=m_wSide/2; point.y=-m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);
	//1
   	point.x=m_wSide/2; point.y=m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);

	AddPoligon(pPoligon);
	//  
	pPoligon=new C3DPoligon(BasePoint);
	if(!pPoligon) return;
	//5
	point.x=-m_wSide/2; point.y=m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);
	//6
	point.x=-m_wSide/2; point.y=-m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);
	//7
	point.x=-m_wSide/2; point.y=-m_wSide/2; point.z=m_wSide/2;
	pPoligon->AddPoint(point);
	//8
	point.x=-m_wSide/2; point.y=m_wSide/2; point.z=m_wSide/2;
	pPoligon->AddPoint(point);
	//5
   	point.x=-m_wSide/2; point.y=m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);

   	AddPoligon(pPoligon);
   
   //
   	pPoligon=new C3DPoligon(BasePoint);
	if(!pPoligon) return;

	//1
   	point.x=m_wSide/2; point.y=m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);
	//4
	point.x=m_wSide/2; point.y=-m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);
	//6
	point.x=-m_wSide/2; point.y=-m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);
	//5
   	point.x=-m_wSide/2; point.y=m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);
	//1
   	point.x=m_wSide/2; point.y=m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);

	AddPoligon(pPoligon);

	//
	pPoligon=new C3DPoligon(BasePoint);
	if(!pPoligon) return;
	//2
	point.x=m_wSide/2; point.y=m_wSide/2; point.z=m_wSide/2;
	pPoligon->AddPoint(point);
	//8
	point.x=-m_wSide/2; point.y=m_wSide/2; point.z=m_wSide/2;
	pPoligon->AddPoint(point);
	//7
	point.x=-m_wSide/2; point.y=-m_wSide/2; point.z=m_wSide/2;
	pPoligon->AddPoint(point);
	//3
	point.x=m_wSide/2; point.y=-m_wSide/2; point.z=m_wSide/2;
	pPoligon->AddPoint(point);
	//2
	point.x=m_wSide/2; point.y=m_wSide/2; point.z=m_wSide/2;
	pPoligon->AddPoint(point);
	
	AddPoligon(pPoligon);

	//
	pPoligon=new C3DPoligon(BasePoint);
	if(!pPoligon) return;
	//4
	point.x=m_wSide/2; point.y=-m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);
	//3
	point.x=m_wSide/2; point.y=-m_wSide/2; point.z=m_wSide/2;
	pPoligon->AddPoint(point);
	//7
	point.x=-m_wSide/2; point.y=-m_wSide/2; point.z=m_wSide/2;
	pPoligon->AddPoint(point);
	//6
	point.x=-m_wSide/2; point.y=-m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);
	//4
	point.x=m_wSide/2; point.y=-m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);

	pPoligon->SetColor(RGB(0, 255, 0));
	AddPoligon(pPoligon);
	
	//
	pPoligon=new C3DPoligon(BasePoint);
	if(!pPoligon) return;
	//1
   	point.x=m_wSide/2; point.y=m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);
	//5
   	point.x=-m_wSide/2; point.y=m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);
	//8
	point.x=-m_wSide/2; point.y=m_wSide/2; point.z=m_wSide/2;
	pPoligon->AddPoint(point);
	//2
	point.x=m_wSide/2; point.y=m_wSide/2; point.z=m_wSide/2;
	pPoligon->AddPoint(point);
	//1
   	point.x=m_wSide/2; point.y=m_wSide/2; point.z=-m_wSide/2;
	pPoligon->AddPoint(point);

	pPoligon->SetColor(RGB(255, 0, 0));
	AddPoligon(pPoligon);
	// 
	MakeProjection();
};
